# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.i_asset_input import IAssetInput
from typing import Optional, Set
from typing_extensions import Self

class CreateDatapointModel(BaseModel):
    """
    The body request for creating a new datapoint
    """ # noqa: E501
    asset: IAssetInput = Field(description="The asset within the datapoint")
    context: Optional[StrictStr] = Field(default=None, description="An additional context to show the users when solving the rapid.")
    context_asset: Optional[IAssetInput] = Field(default=None, description="ContextAsset", alias="contextAsset")
    sort_index: Optional[StrictInt] = Field(default=None, description="The sort index represents the order of the datapoint in the dataset", alias="sortIndex")
    group: Optional[StrictStr] = Field(default=None, description="The group a datapoint belongs to.")
    transcription: Optional[StrictStr] = Field(default=None, description="For audio or video assets, an optional transcription of the content.")
    private_metadata: Optional[Dict[str, StrictStr]] = Field(default=None, description="Private metadata for internal tracking. Not displayed to users.", alias="privateMetadata")
    __properties: ClassVar[List[str]] = ["asset", "context", "contextAsset", "sortIndex", "group", "transcription", "privateMetadata"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateDatapointModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of asset
        if self.asset:
            _dict['asset'] = self.asset.to_dict()
        # override the default output from pydantic by calling `to_dict()` of context_asset
        if self.context_asset:
            _dict['contextAsset'] = self.context_asset.to_dict()
        # set to None if context (nullable) is None
        # and model_fields_set contains the field
        if self.context is None and "context" in self.model_fields_set:
            _dict['context'] = None

        # set to None if group (nullable) is None
        # and model_fields_set contains the field
        if self.group is None and "group" in self.model_fields_set:
            _dict['group'] = None

        # set to None if transcription (nullable) is None
        # and model_fields_set contains the field
        if self.transcription is None and "transcription" in self.model_fields_set:
            _dict['transcription'] = None

        # set to None if private_metadata (nullable) is None
        # and model_fields_set contains the field
        if self.private_metadata is None and "private_metadata" in self.model_fields_set:
            _dict['privateMetadata'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateDatapointModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "asset": IAssetInput.from_dict(obj["asset"]) if obj.get("asset") is not None else None,
            "context": obj.get("context"),
            "contextAsset": IAssetInput.from_dict(obj["contextAsset"]) if obj.get("contextAsset") is not None else None,
            "sortIndex": obj.get("sortIndex"),
            "group": obj.get("group"),
            "transcription": obj.get("transcription"),
            "privateMetadata": obj.get("privateMetadata")
        })
        return _obj





# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class RapidIssue(str, Enum):
    """
    A streamlined enum representing common issues.
    """

    """
    allowed enum values
    """
    OTHER = 'Other'
    CANNOTSUBMIT = 'CannotSubmit'
    NOASSET = 'NoAsset'
    INAPPROPRIATE = 'Inappropriate'
    NOCORRECTOPTION = 'NoCorrectOption'
    WRONGLANGUAGE = 'WrongLanguage'
    DONOTUNDERSTAND = 'DoNotUnderstand'
    DONOTCARE = 'DoNotCare'
    NOTSEEOPTIONSORMEDIADIDNTLOAD = 'NotSeeOptionsOrMediaDidntLoad'
    MYANSWERISCORRECT = 'MyAnswerIsCorrect'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of RapidIssue from a JSON string"""
        return cls(json.loads(json_str))





from __future__ import annotations

from rapidata.rapidata_client.referee._base_referee import Referee
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from rapidata.api_client.models.i_referee_model import IRefereeModel


class EarlyStoppingReferee(Referee):
    """A referee that stops the task when confidence in the winning category exceeds a threshold.

    This referee implements an early stopping mechanism for classification & compare tasks.
    It terminates the task when the confidence in the leading category surpasses
    a specified threshold or when the maximum number of votes is reached.

    The threshold behaves logarithmically, meaning small increments (e.g., from 0.99
    to 0.999) can significantly impact the stopping criteria.

    This referee is supported for the classification and compare tasks (in compare,
    the two options are treated as the categories).

    Args:
        threshold (float, optional): The confidence threshold for early stopping.
            Defaults to 0.999.
        max_responses (int, optional): The maximum number of votes allowed
            before stopping. Defaults to 100.

    Attributes:
        threshold (float): The confidence threshold for early stopping.
        max_responses (int): The maximum number of votes allowed before stopping.
    """

    def __init__(self, threshold: float = 0.999, max_responses: int = 100):
        if threshold <= 0 or threshold >= 1:
            raise ValueError("The threshold must be between 0 and 1.")
        if max_responses < 1:
            raise ValueError("The number of responses must be greater than 0.")

        self.threshold = threshold
        self.max_responses = max_responses

    def _to_dict(self):
        return {
            "_t": "ProbabilisticAttachCategoryRefereeConfig",
            "threshold": self.threshold,
            "maxVotes": self.max_responses,
        }

    def _to_model(self) -> IRefereeModel:
        from rapidata.api_client.models.i_referee_model_early_stopping_referee_model import (
            IRefereeModelEarlyStoppingRefereeModel,
        )
        from rapidata.api_client.models.i_referee_model import IRefereeModel

        return IRefereeModel(
            actual_instance=IRefereeModelEarlyStoppingRefereeModel(
                _t="EarlyStoppingReferee",
                threshold=self.threshold,
                maxVotes=self.max_responses,
            )
        )

    def __str__(self) -> str:
        return f"EarlyStoppingReferee(threshold={self.threshold}, max_responses={self.max_responses})"

    def __repr__(self) -> str:
        return self.__str__()



# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from rapidata.api_client.models.feature_flag import FeatureFlag
from typing import Optional, Set
from typing_extensions import Self

class CreateFlowEndpointInput(BaseModel):
    """
    CreateFlowEndpointInput
    """ # noqa: E501
    name: StrictStr = Field(description="The name of the ranking flow.")
    criteria: StrictStr = Field(description="The ranking criteria used to compare items.")
    validation_set_id: Optional[StrictStr] = Field(default=None, description="Optional ID of the validation set to use.", alias="validationSetId")
    starting_elo: Optional[StrictInt] = Field(default=None, description="Initial Elo rating for new items. Defaults to 1200.", alias="startingElo")
    k_factor: Optional[StrictInt] = Field(default=None, description="K-factor controlling Elo rating sensitivity. Defaults to 40.", alias="kFactor")
    scaling_factor: Optional[StrictInt] = Field(default=None, description="Scaling factor for Elo probability calculation. Defaults to 400.", alias="scalingFactor")
    max_responses: Optional[StrictInt] = Field(default=None, description="Maximum number of responses per comparison.", alias="maxResponses")
    serve_to_response_ratio: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Ratio of concurrent serves to max responses. When set, limits serving to avoid over-collection.", alias="serveToResponseRatio")
    serve_timeout_seconds: Optional[StrictInt] = Field(default=None, description="Time in seconds a user has to submit an answer after loading the task. When set, overrides the global default.", alias="serveTimeoutSeconds")
    min_responses: Optional[StrictInt] = Field(default=None, description="Minimum number of responses per comparison. Defaults to 1.", alias="minResponses")
    responses_required: Optional[StrictInt] = Field(default=None, description="Deprecated. Use MaxResponses instead.", alias="responsesRequired")
    feature_flags: Optional[List[FeatureFlag]] = Field(default=None, alias="featureFlags")
    __properties: ClassVar[List[str]] = ["name", "criteria", "validationSetId", "startingElo", "kFactor", "scalingFactor", "maxResponses", "serveToResponseRatio", "serveTimeoutSeconds", "minResponses", "responsesRequired", "featureFlags"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateFlowEndpointInput from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if validation_set_id (nullable) is None
        # and model_fields_set contains the field
        if self.validation_set_id is None and "validation_set_id" in self.model_fields_set:
            _dict['validationSetId'] = None

        # set to None if feature_flags (nullable) is None
        # and model_fields_set contains the field
        if self.feature_flags is None and "feature_flags" in self.model_fields_set:
            _dict['featureFlags'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateFlowEndpointInput from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "criteria": obj.get("criteria"),
            "validationSetId": obj.get("validationSetId"),
            "startingElo": obj.get("startingElo"),
            "kFactor": obj.get("kFactor"),
            "scalingFactor": obj.get("scalingFactor"),
            "maxResponses": obj.get("maxResponses"),
            "serveToResponseRatio": obj.get("serveToResponseRatio"),
            "serveTimeoutSeconds": obj.get("serveTimeoutSeconds"),
            "minResponses": obj.get("minResponses"),
            "responsesRequired": obj.get("responsesRequired"),
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Rapidata Asset API

    The API for the Rapidata Asset service

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from rapidata.api_client.models.feature_flag import FeatureFlag
from rapidata.api_client.models.i_asset_model import IAssetModel
from rapidata.api_client.models.i_rapid_payload import IRapidPayload
from rapidata.api_client.models.i_referee_info import IRefereeInfo
from rapidata.api_client.models.i_validation_truth import IValidationTruth
from rapidata.api_client.models.rapid_state import RapidState
from typing import Optional, Set
from typing_extensions import Self

class RapidModel(BaseModel):
    """
    RapidModel
    """ # noqa: E501
    id: StrictStr
    payload: IRapidPayload
    referee: IRefereeInfo
    asset: IAssetModel
    state: RapidState
    has_responses: StrictBool = Field(alias="hasResponses")
    should_accept_incorrect: StrictBool = Field(alias="shouldAcceptIncorrect")
    truth: Optional[IValidationTruth]
    explanation: Optional[StrictStr]
    random_correct_probability: Union[StrictFloat, StrictInt] = Field(alias="randomCorrectProbability")
    key: Optional[StrictStr]
    completed_at: Optional[datetime] = Field(alias="completedAt")
    feature_flags: List[FeatureFlag] = Field(alias="featureFlags")
    __properties: ClassVar[List[str]] = ["id", "payload", "referee", "asset", "state", "hasResponses", "shouldAcceptIncorrect", "truth", "explanation", "randomCorrectProbability", "key", "completedAt", "featureFlags"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RapidModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of payload
        if self.payload:
            _dict['payload'] = self.payload.to_dict()
        # override the default output from pydantic by calling `to_dict()` of referee
        if self.referee:
            _dict['referee'] = self.referee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of asset
        if self.asset:
            _dict['asset'] = self.asset.to_dict()
        # override the default output from pydantic by calling `to_dict()` of truth
        if self.truth:
            _dict['truth'] = self.truth.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in feature_flags (list)
        _items = []
        if self.feature_flags:
            for _item_feature_flags in self.feature_flags:
                if _item_feature_flags:
                    _items.append(_item_feature_flags.to_dict())
            _dict['featureFlags'] = _items
        # set to None if truth (nullable) is None
        # and model_fields_set contains the field
        if self.truth is None and "truth" in self.model_fields_set:
            _dict['truth'] = None

        # set to None if explanation (nullable) is None
        # and model_fields_set contains the field
        if self.explanation is None and "explanation" in self.model_fields_set:
            _dict['explanation'] = None

        # set to None if key (nullable) is None
        # and model_fields_set contains the field
        if self.key is None and "key" in self.model_fields_set:
            _dict['key'] = None

        # set to None if completed_at (nullable) is None
        # and model_fields_set contains the field
        if self.completed_at is None and "completed_at" in self.model_fields_set:
            _dict['completedAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RapidModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "payload": IRapidPayload.from_dict(obj["payload"]) if obj.get("payload") is not None else None,
            "referee": IRefereeInfo.from_dict(obj["referee"]) if obj.get("referee") is not None else None,
            "asset": IAssetModel.from_dict(obj["asset"]) if obj.get("asset") is not None else None,
            "state": obj.get("state"),
            "hasResponses": obj.get("hasResponses"),
            "shouldAcceptIncorrect": obj.get("shouldAcceptIncorrect"),
            "truth": IValidationTruth.from_dict(obj["truth"]) if obj.get("truth") is not None else None,
            "explanation": obj.get("explanation"),
            "randomCorrectProbability": obj.get("randomCorrectProbability"),
            "key": obj.get("key"),
            "completedAt": obj.get("completedAt"),
            "featureFlags": [FeatureFlag.from_dict(_item) for _item in obj["featureFlags"]] if obj.get("featureFlags") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from rapidata.api_client.models.attach_category_result import AttachCategoryResult
from rapidata.api_client.models.bounding_box_result import BoundingBoxResult
from rapidata.api_client.models.compare_result import CompareResult
from rapidata.api_client.models.free_text_result import FreeTextResult
from rapidata.api_client.models.line_result import LineResult
from rapidata.api_client.models.locate_result import LocateResult
from rapidata.api_client.models.named_entity_result import NamedEntityResult
from rapidata.api_client.models.polygon_result import PolygonResult
from rapidata.api_client.models.scrub_result import ScrubResult
from rapidata.api_client.models.skip_result import SkipResult
from rapidata.api_client.models.transcription_result import TranscriptionResult
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

GETRESPONSESFORRAPIDRESULTRESPONSERESULT_ONE_OF_SCHEMAS = ["AttachCategoryResult", "BoundingBoxResult", "CompareResult", "FreeTextResult", "LineResult", "LocateResult", "NamedEntityResult", "PolygonResult", "ScrubResult", "SkipResult", "TranscriptionResult"]

class GetResponsesForRapidResultResponseResult(BaseModel):
    """
    GetResponsesForRapidResultResponseResult
    """
    # data type: TranscriptionResult
    oneof_schema_1_validator: Optional[TranscriptionResult] = None
    # data type: ScrubResult
    oneof_schema_2_validator: Optional[ScrubResult] = None
    # data type: PolygonResult
    oneof_schema_3_validator: Optional[PolygonResult] = None
    # data type: NamedEntityResult
    oneof_schema_4_validator: Optional[NamedEntityResult] = None
    # data type: LocateResult
    oneof_schema_5_validator: Optional[LocateResult] = None
    # data type: LineResult
    oneof_schema_6_validator: Optional[LineResult] = None
    # data type: FreeTextResult
    oneof_schema_7_validator: Optional[FreeTextResult] = None
    # data type: CompareResult
    oneof_schema_8_validator: Optional[CompareResult] = None
    # data type: SkipResult
    oneof_schema_9_validator: Optional[SkipResult] = None
    # data type: AttachCategoryResult
    oneof_schema_10_validator: Optional[AttachCategoryResult] = None
    # data type: BoundingBoxResult
    oneof_schema_11_validator: Optional[BoundingBoxResult] = None
    actual_instance: Optional[Union[AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult]] = None
    one_of_schemas: Set[str] = { "AttachCategoryResult", "BoundingBoxResult", "CompareResult", "FreeTextResult", "LineResult", "LocateResult", "NamedEntityResult", "PolygonResult", "ScrubResult", "SkipResult", "TranscriptionResult" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )


    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        instance = GetResponsesForRapidResultResponseResult.model_construct()
        error_messages = []
        match = 0
        # validate data type: TranscriptionResult
        if not isinstance(v, TranscriptionResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `TranscriptionResult`")
        else:
            match += 1
        # validate data type: ScrubResult
        if not isinstance(v, ScrubResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ScrubResult`")
        else:
            match += 1
        # validate data type: PolygonResult
        if not isinstance(v, PolygonResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `PolygonResult`")
        else:
            match += 1
        # validate data type: NamedEntityResult
        if not isinstance(v, NamedEntityResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `NamedEntityResult`")
        else:
            match += 1
        # validate data type: LocateResult
        if not isinstance(v, LocateResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocateResult`")
        else:
            match += 1
        # validate data type: LineResult
        if not isinstance(v, LineResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LineResult`")
        else:
            match += 1
        # validate data type: FreeTextResult
        if not isinstance(v, FreeTextResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `FreeTextResult`")
        else:
            match += 1
        # validate data type: CompareResult
        if not isinstance(v, CompareResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CompareResult`")
        else:
            match += 1
        # validate data type: SkipResult
        if not isinstance(v, SkipResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SkipResult`")
        else:
            match += 1
        # validate data type: AttachCategoryResult
        if not isinstance(v, AttachCategoryResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AttachCategoryResult`")
        else:
            match += 1
        # validate data type: BoundingBoxResult
        if not isinstance(v, BoundingBoxResult):
            error_messages.append(f"Error! Input type `{type(v)}` is not `BoundingBoxResult`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in GetResponsesForRapidResultResponseResult with oneOf schemas: AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in GetResponsesForRapidResultResponseResult with oneOf schemas: AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        match = 0

        # deserialize data into TranscriptionResult
        try:
            instance.actual_instance = TranscriptionResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ScrubResult
        try:
            instance.actual_instance = ScrubResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into PolygonResult
        try:
            instance.actual_instance = PolygonResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into NamedEntityResult
        try:
            instance.actual_instance = NamedEntityResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocateResult
        try:
            instance.actual_instance = LocateResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LineResult
        try:
            instance.actual_instance = LineResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into FreeTextResult
        try:
            instance.actual_instance = FreeTextResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CompareResult
        try:
            instance.actual_instance = CompareResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SkipResult
        try:
            instance.actual_instance = SkipResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into AttachCategoryResult
        try:
            instance.actual_instance = AttachCategoryResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into BoundingBoxResult
        try:
            instance.actual_instance = BoundingBoxResult.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into GetResponsesForRapidResultResponseResult with oneOf schemas: AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into GetResponsesForRapidResultResponseResult with oneOf schemas: AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AttachCategoryResult, BoundingBoxResult, CompareResult, FreeTextResult, LineResult, LocateResult, NamedEntityResult, PolygonResult, ScrubResult, SkipResult, TranscriptionResult]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())





# coding: utf-8

"""
    Rapidata.Dataset

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from rapidata.api_client.models.completed_rapid_model import CompletedRapidModel
from rapidata.api_client.models.get_workflow_progress_result import GetWorkflowProgressResult
from rapidata.api_client.models.in_progress_rapid_model import InProgressRapidModel
from rapidata.api_client.models.not_started_rapid_model import NotStartedRapidModel
from typing import Optional, Set
from typing_extensions import Self

class GetAttachCategoryWorkflowResultOverviewResult(BaseModel):
    """
    GetAttachCategoryWorkflowResultOverviewResult
    """ # noqa: E501
    state: StrictStr
    progress: GetWorkflowProgressResult
    rapid_type: StrictStr = Field(alias="rapidType")
    not_started_rapids: List[NotStartedRapidModel] = Field(alias="notStartedRapids")
    hidden_not_started_count: StrictInt = Field(alias="hiddenNotStartedCount")
    in_progress_rapids: List[InProgressRapidModel] = Field(alias="inProgressRapids")
    hidden_in_progress_rapids_count: StrictInt = Field(alias="hiddenInProgressRapidsCount")
    completed_rapids: List[CompletedRapidModel] = Field(alias="completedRapids")
    hidden_completed_rapids_count: StrictInt = Field(alias="hiddenCompletedRapidsCount")
    result_type: Optional[StrictStr] = Field(default=None, alias="resultType")
    occurrences_by_category: Dict[str, StrictInt] = Field(alias="occurrencesByCategory")
    __properties: ClassVar[List[str]] = ["state", "progress", "rapidType", "notStartedRapids", "hiddenNotStartedCount", "inProgressRapids", "hiddenInProgressRapidsCount", "completedRapids", "hiddenCompletedRapidsCount", "resultType", "occurrencesByCategory"]

    @field_validator('state')
    def state_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['Created', 'Started', 'Labeling', 'Paused', 'Done', 'Failed']):
            raise ValueError("must be one of enum values ('Created', 'Started', 'Labeling', 'Paused', 'Done', 'Failed')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetAttachCategoryWorkflowResultOverviewResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "result_type",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of progress
        if self.progress:
            _dict['progress'] = self.progress.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in not_started_rapids (list)
        _items = []
        if self.not_started_rapids:
            for _item_not_started_rapids in self.not_started_rapids:
                if _item_not_started_rapids:
                    _items.append(_item_not_started_rapids.to_dict())
            _dict['notStartedRapids'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in in_progress_rapids (list)
        _items = []
        if self.in_progress_rapids:
            for _item_in_progress_rapids in self.in_progress_rapids:
                if _item_in_progress_rapids:
                    _items.append(_item_in_progress_rapids.to_dict())
            _dict['inProgressRapids'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in completed_rapids (list)
        _items = []
        if self.completed_rapids:
            for _item_completed_rapids in self.completed_rapids:
                if _item_completed_rapids:
                    _items.append(_item_completed_rapids.to_dict())
            _dict['completedRapids'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetAttachCategoryWorkflowResultOverviewResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "state": obj.get("state"),
            "progress": GetWorkflowProgressResult.from_dict(obj["progress"]) if obj.get("progress") is not None else None,
            "rapidType": obj.get("rapidType"),
            "notStartedRapids": [NotStartedRapidModel.from_dict(_item) for _item in obj["notStartedRapids"]] if obj.get("notStartedRapids") is not None else None,
            "hiddenNotStartedCount": obj.get("hiddenNotStartedCount"),
            "inProgressRapids": [InProgressRapidModel.from_dict(_item) for _item in obj["inProgressRapids"]] if obj.get("inProgressRapids") is not None else None,
            "hiddenInProgressRapidsCount": obj.get("hiddenInProgressRapidsCount"),
            "completedRapids": [CompletedRapidModel.from_dict(_item) for _item in obj["completedRapids"]] if obj.get("completedRapids") is not None else None,
            "hiddenCompletedRapidsCount": obj.get("hiddenCompletedRapidsCount"),
            "resultType": obj.get("resultType"),
            "occurrencesByCategory": obj.get("occurrencesByCategory")
        })
        return _obj


